"""
note that if there are ambiguities in dirUriMap, you'll get
undefined results that depend on python's items() order.

In this example:
  http://example.com/a/b/c <-> /my/dir/b/c.n3
  bbbbbbbbbbbbbbbbbbbb mmm     ddddddd mmm
The code naming goes like this:
  b is baseUri
  m is middle
  d is dirpath

DirUriMap is a dict of dirPath -> baseUri.

"""
from pathlib import Path
from typing import Dict, Union

from rdflib import URIRef, Namespace

DirUriMap = Dict[Path, Union[URIRef, Namespace]]


def _pathStartsWith(fullPath: Path, possiblePrefix: Path) -> bool:
    return str(fullPath).startswith(str(possiblePrefix))


def uriFromFile(dirUriMap: DirUriMap, filename: Path) -> URIRef:
    if not str(filename).endswith('.n3'):
        raise ValueError(filename)
    for dirPath, baseUri in dirUriMap.items():
        if _pathStartsWith(filename, dirPath):
            middle = str(filename)[len(str(dirPath)):-len('.n3')].lstrip('/')
            return URIRef(baseUri.rstrip('/') + '/' + middle)
    raise ValueError("filename %s doesn't start with any of %s" % (filename, list(dirUriMap.keys())))


def fileForUri(dirUriMap: DirUriMap, ctx: URIRef) -> Path:
    for dirPath, prefix in dirUriMap.items():
        if ctx.startswith(prefix):
            middle = ctx[len(prefix):].lstrip('/')
            if middle.endswith('/'):
                raise NotImplementedError(f'no file mapped to {ctx=}')
            return dirPath / (middle + '.n3')
    raise ValueError("don't know what filename to use for %s" % ctx)


def correctToTopdirPrefix(dirUriMap: DirUriMap, inFile: Path) -> Path:
    # can't remember what this is for, so it's a no-op to hopefully flush out the bugs
    return inFile
    if not any(str(inFile).startswith(str(prefix)) for prefix in dirUriMap):
        for prefix in dirUriMap:
            prefixAbs = prefix.absolute()
            if _pathStartsWith(inFile, prefixAbs):
                inFile = Path(str(prefix) + str(inFile)[len(str(prefixAbs)):])
                break
        else:
            raise ValueError("can't correct %s to start with one of %s" % (inFile, list(dirUriMap.keys())))
    return inFile
