import asyncio
import logging
from pathlib import Path

import pytest
from rdflib import Namespace

from .watched_graphs import WatchedGraphs

log = logging.getLogger()

EX = Namespace('http://example.com/')


async def assert_no_events_coming(q: asyncio.Queue):
    """
    see if inotify makes an event or stays quiet
    """
    inotify_max_wait_secs = 0.1
    with pytest.raises(asyncio.TimeoutError):
        # no events from the empty dir
        ev = await asyncio.wait_for(q.get(), timeout=inotify_max_wait_secs)
        log.error(f'expected no events; got {ev}', stacklevel=2)


@pytest.mark.asyncio
async def test_make_graph_from_file(tmpdir):
    root = Path(tmpdir.mkdir("root"))
    someFile = root / 'hello.n3'
    someFile.write_text("# some n3")

    wg = WatchedGraphs({root: EX}, addlPrefixes={})

    ev = await asyncio.wait_for(wg.graphEditEvents.get(), timeout=1.0)
    assert ev.uri == EX['hello']
    assert list(ev.content) == []

    await assert_no_events_coming(wg.graphEditEvents)
    wg.cancel()
